@@ -1,7 +1,7 @@
 /*
- * oAuth string functions in POSIX-C.
+ * OAuth string functions in POSIX-C.
  *
- * Copyright 2007, 2008, 2009 Robin Gareus <robin@gareus.org>
+ * Copyright 2007-2010 Robin Gareus <robin@gareus.org>
  * 
  * The base64 functions are by Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * and un/escape_url() was inspired by libcurl's curl_escape under ISC-license
@@ -39,7 +39,6 @@
 #include <time.h>
 #include <math.h>
 #include <ctype.h> // isxdigit
-#include <openssl/hmac.h>
 
 #include "xmalloc.h"
 #include "oauth.h"
@@ -131,7 +130,7 @@ char *oauth_encode_base64(int size, const unsigned char *src) {
  * 'dest'. 
  *
  * @param dest Pointer to memory for holding the decoded string.
- * Must be large enough to recieve the decoded string.
+ * Must be large enough to receive the decoded string.
  * @param src A base64 encoded string.
  * @return the length of the decoded string if decode
  * succeeded otherwise 0.
@@ -276,32 +275,6 @@ char *oauth_url_unescape(const char *string, size_t *olen) {
   return ns;
 }
 
-/**
- * returns base64 encoded HMAC-SHA1 signature for
- * given message and key.
- * both data and key need to be urlencoded.
- *
- * the returned string needs to be freed by the caller
- *
- * @param m message to be signed
- * @param k key used for signing
- * @return signature string.
- */
-char *oauth_sign_hmac_sha1 (const char *m, const char *k) {
-  return(oauth_sign_hmac_sha1_raw (m, strlen(m), k, strlen(k)));
-}
-
-char *oauth_sign_hmac_sha1_raw (const char *m, const size_t ml, const char *k, const size_t kl) {
-  unsigned char result[EVP_MAX_MD_SIZE];
-  unsigned int resultlen = 0;
-  
-  HMAC(EVP_sha1(), k, kl, 
-      (unsigned char*) m, ml,
-      result, &resultlen);
-
-  return(oauth_encode_base64(resultlen, result));
-}
-
 /**
  * returns plaintext signature for the given key.
  *
@@ -315,97 +288,6 @@ char *oauth_sign_plaintext (const char *m, const char *k) {
   return(oauth_url_escape(k));
 }
 
-#include <openssl/evp.h>
-#include <openssl/x509.h>
-#include <openssl/x509v3.h>
-#include <openssl/ssl.h>
-
-/**
- * returns RSA-SHA1 signature for given data.
- * the returned signature needs to be freed by the caller.
- *
- * @param m message to be signed
- * @param k private-key PKCS and Base64-encoded 
- * @return base64 encoded signature string.
- */
-char *oauth_sign_rsa_sha1 (const char *m, const char *k) {
-  unsigned char *sig = NULL;
-  unsigned char *passphrase = NULL;
-  unsigned int len=0;
-  EVP_MD_CTX md_ctx;
-
-  EVP_PKEY *pkey;
-  BIO *in;
-  in = BIO_new_mem_buf((unsigned char*) k, strlen(k));
-  pkey = PEM_read_bio_PrivateKey(in, NULL, 0, passphrase); // generate sign
-  BIO_free(in);
-
-  if (pkey == NULL) {
-  //fprintf(stderr, "liboauth/ssl: can not read private key\n");
-    return xstrdup("liboauth/ssl: can not read private key");
-  }
-
-  len = EVP_PKEY_size(pkey);
-  sig = (unsigned char*)xmalloc((len+1)*sizeof(char));
-
-  EVP_SignInit(&md_ctx, EVP_sha1());
-  EVP_SignUpdate(&md_ctx, m, strlen(m));
-  if (EVP_SignFinal (&md_ctx, sig, &len, pkey)) {
-    char *tmp;
-    sig[len] = '\0';
-    tmp = oauth_encode_base64(len,sig);
-    OPENSSL_free(sig);
-    EVP_PKEY_free(pkey);
-    return tmp;
-  }
-  return xstrdup("liboauth/ssl: rsa-sha1 signing failed");
-}
-
-/**
- * verify RSA-SHA1 signature.
- *
- * returns the output of EVP_VerifyFinal() for a given message,
- * cert/pubkey and signature
- *
- * @param m message to be verified
- * @param c public-key or x509 certificate
- * @param s base64 encoded signature
- * @return 1 for a correct signature, 0 for failure and -1 if some other error occurred
- */
-int oauth_verify_rsa_sha1 (const char *m, const char *c, const char *s) {
-  EVP_MD_CTX md_ctx;
-  EVP_PKEY *pkey;
-  BIO *in;
-  X509 *cert = NULL;
-  unsigned char *b64d;
-  int slen, err;
-
-  in = BIO_new_mem_buf((unsigned char*)c, strlen(c));
-  cert = PEM_read_bio_X509(in, NULL, 0, NULL);
-  if (cert)  {
-    pkey = (EVP_PKEY *) X509_get_pubkey(cert); 
-    X509_free(cert);
-  } else {
-    pkey = PEM_read_bio_PUBKEY(in, NULL, 0, NULL);
-  }
-  BIO_free(in);
-  if (pkey == NULL) {
-  //fprintf(stderr, "could not read cert/pubkey.\n");
-    return -2;
-  }
-
-  b64d= (unsigned char*) xmalloc(sizeof(char)*strlen(s));
-  slen = oauth_decode_base64(b64d, s);
-
-  EVP_VerifyInit(&md_ctx, EVP_sha1());
-  EVP_VerifyUpdate(&md_ctx, m, strlen(m));
-  err = EVP_VerifyFinal(&md_ctx, b64d, slen, pkey);
-  EVP_MD_CTX_cleanup(&md_ctx);
-  EVP_PKEY_free(pkey);
-  free(b64d);
-  return (err);
-}
-
 /**
  * encode strings and concatenate with '&' separator.
  * The number of strings to be concatenated must be
@@ -529,7 +411,7 @@ int oauth_split_url_parameters(const char *url, char ***argv) {
  *
  */
 char *oauth_serialize_url (int argc, int start, char **argv) {
-  return oauth_serialize_url_sep( argc, start, argv, "&");
+  return oauth_serialize_url_sep( argc, start, argv, "&", 0);
 }
 
 /**
@@ -539,9 +421,13 @@ char *oauth_serialize_url (int argc, int start, char **argv) {
  * @param start element in the array at which to start concatenating.
  * @param argv parameter-array to concatenate.
  * @param sep separator for parameters (usually "&") 
+ * @param mod - bitwise modifiers: 
+ *   1: skip all values that start with "oauth_"
+ *   2: skip all values that don't start with "oauth_"
+ *   4: add double quotation marks around values (use with sep=", " to generate HTTP Authorization header).
  * @return url string needs to be freed by the caller.
  */
-char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep) {
+char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep, int mod) {
   char  *tmp, *t1;
   int i;
   int  first=0;
@@ -550,6 +436,9 @@ char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep) {
   *query='\0';
   for(i=start; i< argc; i++) {
     int len = 0;
+    if ((mod&1)==1 && (strncmp(argv[i],"oauth_",6) == 0 || strncmp(argv[i],"x_oauth_",8) == 0) ) continue;
+    if ((mod&2)==2 && (strncmp(argv[i],"oauth_",6) != 0 && strncmp(argv[i],"x_oauth_",8) != 0) && i!=0) continue;
+
     if (query) len+=strlen(query);
 
     if (i==start && i==0 && strstr(argv[i], ":/")) {
@@ -567,9 +456,11 @@ char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep) {
       tmp = oauth_url_escape(argv[i]);
       *t1='=';
       t1 = oauth_url_escape((t1+1));
-      tmp=(char*) xrealloc(tmp,(strlen(tmp)+strlen(t1)+2)*sizeof(char));
+      tmp=(char*) xrealloc(tmp,(strlen(tmp)+strlen(t1)+2+(mod&4?2:0))*sizeof(char));
       strcat(tmp,"=");
+      if (mod&4) strcat(tmp,"\"");
       strcat(tmp,t1);
+      if (mod&4) strcat(tmp,"\"");
       free(t1);
       len+=strlen(tmp);
     }
@@ -609,6 +500,8 @@ char *oauth_serialize_url_parameters (int argc, char **argv) {
  *
  * @return zero terminated random string.
  */
+#if !defined HAVE_OPENSSL_HMAC_H && !defined USE_NSS
+/* pre liboauth-0.7.2 and possible future versions that don't use OpenSSL or NSS */
 char *oauth_gen_nonce() {
   char *nc;
   static int rndinit = 1;
@@ -631,6 +524,41 @@ char *oauth_gen_nonce() {
   nc[i]='\0';
   return (nc);
 }
+#else // OpenSSL or NSS random number generator
+#ifdef USE_NSS
+  void oauth_init_nss(); //decladed in hash.c
+#  include "pk11pub.h"
+#  define MY_RAND PK11_GenerateRandom
+#  define MY_SRAND  oauth_init_nss();
+#else
+#  ifdef _GNU_SOURCE
+/* Note: the OpenSSL/GPL exemption stated 
+ * verbosely in hash.c applies to this code as well. */
+#  endif
+#  include <openssl/rand.h>
+#  define MY_RAND RAND_bytes
+#  define MY_SRAND ;
+#endif
+char *oauth_gen_nonce() {
+  char *nc;
+  unsigned char buf;
+  const char *chars = "abcdefghijklmnopqrstuvwxyz"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "0123456789_";
+  unsigned int max = strlen(chars);
+  int i, len;
+
+  MY_SRAND
+  MY_RAND(&buf, 1);
+  len=15+(((short)buf)&0x0f);
+  nc = (char*) xmalloc((len+1)*sizeof(char));
+  for(i=0;i<len; i++) {
+    MY_RAND(&buf, 1);
+    nc[i] = chars[ ((short)buf) % max ];
+  }
+  nc[i]='\0';
+  return (nc);
+}
+#endif
 
 /**
  * string compare function for oauth parameters.
@@ -666,7 +594,11 @@ int oauth_cmpstringp(const void *p1, const void *p2) {
   // if parameter names are equal, sort by value.
   if (t1) t1[0]='='; 
   if (t2) t2[0]='='; 
-  rv=strcmp(t1,t2);
+  if (t1 && t2)        rv=strcmp(t1,t2);
+  else if (!t1 && !t2) rv=0;
+  else if (!t1)        rv=-1;
+  else                 rv=1;
+
   if (v1) free(v1);
   if (v2) free(v2);
   return rv;
@@ -678,7 +610,7 @@ int oauth_cmpstringp(const void *p1, const void *p2) {
  * @param argc parameter array to search
  * @param key key of parameter to check.
  *
- * @return FALSE (0) if array does not contain a paramater with given key, TRUE (1) otherwise.
+ * @return FALSE (0) if array does not contain a parameter with given key, TRUE (1) otherwise.
  */
 int oauth_param_exists(char **argv, int argc, char *key) {
   int i;
@@ -710,7 +642,7 @@ void oauth_add_protocol(int *argcp, char ***argvp,
  ){
   char oarg[1024];
 
-  // add oAuth specific arguments
+  // add OAuth specific arguments
   if (!oauth_param_exists(*argvp,*argcp,"oauth_nonce")) {
     char *tmp;
     snprintf(oarg, 1024, "oauth_nonce=%s", (tmp=oauth_gen_nonce()));
@@ -802,7 +734,7 @@ char *oauth_sign_array (int *argcp, char***argvp,
                             t_key, t_secret);
 }
 
-char *oauth_sign_array2 (int *argcp, char***argvp,
+void oauth_sign_array2_process (int *argcp, char***argvp,
   char **postargs,
   OAuthMethod method, 
   const char *http_method, //< HTTP request method
@@ -814,7 +746,6 @@ char *oauth_sign_array2 (int *argcp, char***argvp,
   char oarg[1024];
   char *query;
   char *okey, *odat, *sign;
-  char *result;
   char *http_request_method;
 
   if (!http_method) {
@@ -826,18 +757,18 @@ char *oauth_sign_array2 (int *argcp, char***argvp,
       http_request_method[i]=toupper(http_request_method[i]);
   }
 
-  // add OAuth protocol parameters
+  // add required OAuth protocol parameters
   oauth_add_protocol(argcp, argvp, method, c_key, t_key);
 
   // sort parameters
   qsort(&(*argvp)[1], (*argcp)-1, sizeof(char *), oauth_cmpstringp);
 
-  // serialize URL
+  // serialize URL - base-url 
   query= oauth_serialize_url_parameters(*argcp, *argvp);
 
   // generate signature
   okey = oauth_catenc(2, c_secret, t_secret);
-  odat = oauth_catenc(3, http_request_method, (*argvp)[0], query);
+  odat = oauth_catenc(3, http_request_method, (*argvp)[0], query); // base-string
   free(http_request_method);
 #ifdef DEBUG_OAUTH
   fprintf (stderr, "\nliboauth: data to sign='%s'\n\n", odat);
@@ -864,6 +795,21 @@ char *oauth_sign_array2 (int *argcp, char***argvp,
   snprintf(oarg, 1024, "oauth_signature=%s",sign);
   oauth_add_param_to_array(argcp, argvp, oarg);
   free(sign);
+  if(query) free(query);
+}
+
+char *oauth_sign_array2 (int *argcp, char***argvp,
+  char **postargs,
+  OAuthMethod method, 
+  const char *http_method, //< HTTP request method
+  const char *c_key, //< consumer key - posted plain text
+  const char *c_secret, //< consumer secret - used as 1st part of secret-key 
+  const char *t_key, //< token key - posted plain text in URL
+  const char *t_secret //< token secret - used as 2st part of secret-key
+  ) {
+
+  char *result;
+  oauth_sign_array2_process(argcp, argvp, postargs, method, http_method, c_key, c_secret, t_key, t_secret);
 
   // build URL params
   result = oauth_serialize_url(*argcp, (postargs?1:0), *argvp);
@@ -873,11 +819,10 @@ char *oauth_sign_array2 (int *argcp, char***argvp,
     result = xstrdup((*argvp)[0]);
   }
 
-  if(query) free(query);
-
   return result;
 }
 
+
 /**
  * free array args
  *
@@ -892,50 +837,13 @@ void oauth_free_array(int *argcp, char ***argvp) {
   if(*argvp) free(*argvp);
 }
 
-/** 
- * http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/drafts/4/spec.html
- */
-char *oauth_body_hash_file(char *filename) {
-  unsigned char fb[BUFSIZ];
-  EVP_MD_CTX ctx;
-  size_t len=0;
-  unsigned char *md;
-  FILE *F= fopen(filename, "r");
-  if (!F) return NULL;
-
-  EVP_MD_CTX_init(&ctx);
-  EVP_DigestInit(&ctx,EVP_sha1());
-  while (!feof(F) && (len=fread(fb,sizeof(char),BUFSIZ, F))>0) {
-    EVP_DigestUpdate(&ctx, fb, len);
-  }
-  fclose(F);
-  len=0;
-  md=(unsigned char*) calloc(EVP_MD_size(EVP_sha1()),sizeof(unsigned char));
-  EVP_DigestFinal(&ctx, md,(unsigned int*) &len);
-  EVP_MD_CTX_cleanup(&ctx);
-  return oauth_body_hash_encode(len, md);
-}
-
-char *oauth_body_hash_data(size_t length, const char *data) {
-  EVP_MD_CTX ctx;
-  size_t len=0;
-  unsigned char *md;
-  md=(unsigned char*) calloc(EVP_MD_size(EVP_sha1()),sizeof(unsigned char));
-  EVP_MD_CTX_init(&ctx);
-  EVP_DigestInit(&ctx,EVP_sha1());
-  EVP_DigestUpdate(&ctx, data, length);
-  EVP_DigestFinal(&ctx, md,(unsigned int*) &len);
-  EVP_MD_CTX_cleanup(&ctx);
-  return oauth_body_hash_encode(len, md);
-}
-
 /**
  * base64 encode digest, free it and return a URL parameter
  * with the oauth_body_hash
  */
 char *oauth_body_hash_encode(size_t len, unsigned char *digest) {
   char *sign=oauth_encode_base64(len,digest);
-  char *sig_url = (char*)malloc(17+strlen(sign));
+  char *sig_url = (char*)xmalloc(17+strlen(sign));
   sprintf(sig_url,"oauth_body_hash=%s", sign);
   free(sign);
   free(digest);
