@@ -1,9 +1,9 @@
 /**
- *  @brief oAuth.net implementation in POSIX-C.
+ *  @brief OAuth.net implementation in POSIX-C.
  *  @file oauth.h
  *  @author Robin Gareus <robin@gareus.org>
  *
- * Copyright 2007, 2008, 2009 Robin Gareus <robin@gareus.org>
+ * Copyright 2007-2010 Robin Gareus <robin@gareus.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,16 +29,16 @@
 
 #ifndef DOXYGEN_IGNORE
 // liboauth version
-#define LIBOAUTH_VERSION "0.6.1"
+#define LIBOAUTH_VERSION "0.8.8"
 #define LIBOAUTH_VERSION_MAJOR  0
-#define LIBOAUTH_VERSION_MINOR  6
-#define LIBOAUTH_VERSION_MICRO  1
+#define LIBOAUTH_VERSION_MINOR  8
+#define LIBOAUTH_VERSION_MICRO  8
 
 //interface revision number
 //http://www.gnu.org/software/libtool/manual/html_node/Updating-version-info.html
-#define LIBOAUTH_CUR  5
+#define LIBOAUTH_CUR  6
 #define LIBOAUTH_REV  0
-#define LIBOAUTH_AGE  5
+#define LIBOAUTH_AGE  6
 #endif
 
 #ifdef __GNUC__
@@ -60,7 +60,7 @@
  */ 
 typedef enum { 
     OA_HMAC=0, ///< use HMAC-SHA1 request signing method
-    OA_RSA, ///< use RSA signature (not implemented)
+    OA_RSA, ///< use RSA signature 
     OA_PLAINTEXT ///< use plain text signature (for testing only)
   } OAuthMethod;
 
@@ -79,7 +79,7 @@ char *oauth_encode_base64(int size, const unsigned char *src);
  * 'dest'. 
  *
  * @param dest Pointer to memory for holding the decoded string.
- * Must be large enough to recieve the decoded string.
+ * Must be large enough to receive the decoded string.
  * @param src A base64 encoded string.
  * @return the length of the decoded string if decode
  * succeeded otherwise 0.
@@ -229,9 +229,13 @@ char *oauth_serialize_url (int argc, int start, char **argv);
  * @param start element in the array at which to start concatenating.
  * @param argv parameter-array to concatenate.
  * @param sep separator for parameters (usually "&") 
+ * @param mod - bitwise modifiers: 
+ *   1: skip all values that start with "oauth_"
+ *   2: skip all values that don't start with "oauth_"
+ *   4: double quotation marks are added around values (use with sep ", " for HTTP Authorization header).
  * @return url string needs to be freed by the caller.
  */
-char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep);
+char *oauth_serialize_url_sep (int argc, int start, char **argv, char *sep, int mod);
 
 /**
  * build a query parameter string from an array.
@@ -270,7 +274,7 @@ int oauth_cmpstringp(const void *p1, const void *p2);
  * @param argc parameter array to search
  * @param key key of parameter to check.
  *
- * @return FALSE (0) if array does not contain a paramater with given key, TRUE (1) otherwise.
+ * @return FALSE (0) if array does not contain a parameter with given key, TRUE (1) otherwise.
  */
 int oauth_param_exists(char **argv, int argc, char *key);
 
@@ -292,7 +296,7 @@ void oauth_add_param_to_array(int *argcp, char ***argvp, const char *addparam);
 void oauth_free_array(int *argcp, char ***argvp);
 
 /**
- * calculate oAuth-signature for a given HTTP request URL, parameters and oauth-tokens.
+ * calculate OAuth-signature for a given HTTP request URL, parameters and oauth-tokens.
  *
  * if 'postargs' is NULL a "GET" request is signed and the 
  * signed URL is returned. Else this fn will modify 'postargs' 
@@ -343,13 +347,64 @@ char *oauth_sign_url (const char *url, char **postargs,
   const char *t_secret //< token secret - used as 2st part of secret-key
   ) attribute_deprecated;
 
+
+/**
+ * the back-end behind by /ref oauth_sign_array2.
+ * however it does not serialize the signed URL again.
+ * The user needs to call /ref oauth_serialize_url (oA)
+ * and /ref oauth_free_array to do so.
+ *
+ * This allows to split parts of the URL to be used for
+ * OAuth HTTP Authorization header:
+ * see http://oauth.net/core/1.0a/#consumer_req_param
+ * the oauthtest2 example code does so.
+ * 
+ *
+ * @param argcp pointer to array length int
+ * @param argvp pointer to array values  
+ * (argv[0]="http://example.org:80/" argv[1]="first=QueryParamater" ..
+ *  the array is modified: fi. oauth_ parameters are added)
+ *  These arrays can be generated with /ref oauth_split_url_parameters
+ *  or /ref oauth_split_post_paramters.
+ *
+ * @param postargs This parameter points to an area where the return value
+ * is stored. If 'postargs' is NULL, no value is stored.
+ *
+ * @param method specify the signature method to use. It is of type 
+ * \ref OAuthMethod and most likely \ref OA_HMAC.
+ *
+ * @param http_method The HTTP request method to use (ie "GET", "PUT",..)
+ * If NULL is given as 'http_method' this defaults to "GET" when 
+ * 'postargs' is also NULL and when postargs is not NULL "POST" is used.
+ *
+ * @param c_key consumer key
+ * @param c_secret consumer secret
+ * @param t_key token key
+ * @param t_secret token secret
+ *
+ * @return void
+ *
+ */
+void oauth_sign_array2_process (int *argcp, char***argvp,
+  char **postargs,
+  OAuthMethod method, 
+  const char *http_method, //< HTTP request method
+  const char *c_key, //< consumer key - posted plain text
+  const char *c_secret, //< consumer secret - used as 1st part of secret-key 
+  const char *t_key, //< token key - posted plain text in URL
+  const char *t_secret //< token secret - used as 2st part of secret-key
+  );
+
 /**
  * same as /ref oauth_sign_url
  * with the url already split into parameter array 
  *
  * @param argcp pointer to array length int
- * @param argvp pointer to array values 
- * (argv[0]="http://example.org:80/" argv[1]="first=QueryParamater" ..)
+ * @param argvp pointer to array values  
+ * (argv[0]="http://example.org:80/" argv[1]="first=QueryParamater" ..
+ *  the array is modified: fi. oauth_ parameters are added)
+ *  These arrays can be generated with /ref oauth_split_url_parameters
+ *  or /ref oauth_split_post_paramters.
  *
  * @param postargs This parameter points to an area where the return value
  * is stored. If 'postargs' is NULL, no value is stored.
@@ -397,7 +452,7 @@ char *oauth_sign_array (int *argcp, char***argvp,
  * The returned string needs to be freed by the calling function.
  *
  * see
- * http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/drafts/4/spec.html
+ * http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/oauth-bodyhash.html
  * 
  * @param filename the filename to calculate the hash for
  *
@@ -413,7 +468,7 @@ char *oauth_body_hash_file(char *filename);
  * passed as argument to \ref oauth_catenc.
  *
  * see
- * http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/drafts/4/spec.html
+ * http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/oauth-bodyhash.html
  * 
  * @param length length of the data parameter in bytes
  * @param data to calculate the hash for
@@ -456,10 +511,10 @@ char *oauth_sign_xmpp (const char *xml,
  * where %%u is replaced with the URL and query parameters.
  *
  * bash & wget example:
- * <tt>export OAUTH_HTTP_CMD="wget -q -U 'liboauth-agent/0.1' '%%u' "</tt>
+ * <tt>export OAUTH_HTTP_CMD="wget -q -U 'liboauth-agent/0.1' '%u' "</tt>
  *
  * WARNING: this is a tentative function. it's convenient and handy for testing
- * or developing oAuth code. But don't rely on this function
+ * or developing OAuth code. But don't rely on this function
  * to become a stable part of this API. It does not do 
  * much error checking or handling for one thing..
  *
@@ -473,6 +528,28 @@ char *oauth_sign_xmpp (const char *xml,
  */
 char *oauth_http_get (const char *u, const char *q);
 
+/**
+ * do a HTTP GET request, wait for it to finish 
+ * and return the content of the reply.
+ *
+ * (requires libcurl)
+ *
+ * This is equivalent to /ref oauth_http_get but allows to
+ * specifiy a custom HTTP header and has
+ * has no support for commandline-curl.
+ *
+ * If liboauth is compiled <b>without</b> libcurl this function
+ * always returns NULL.
+ *
+ * @param u base url to get
+ * @param q query string to send along with the HTTP request or NULL.
+ * @param customheader specify custom HTTP header (or NULL for none)
+ * Multiple header elements can be passed separating them with "\r\n"
+ * @return  In case of an error NULL is returned; otherwise a pointer to the
+ * replied content from HTTP server. latter needs to be freed by caller.
+ */
+char *oauth_http_get2 (const char *u, const char *q, const char *customheader);
+
 
 /**
  * do a HTTP POST request, wait for it to finish 
@@ -487,7 +564,7 @@ char *oauth_http_get (const char *u, const char *q);
  * the URL. 
  *
  * bash & wget example:
- * <tt>export OAUTH_HTTP_CMD="wget -q -U 'liboauth-agent/0.1' --post-data='%%p' '%%u' "</tt>
+ * <tt>export OAUTH_HTTP_CMD="wget -q -U 'liboauth-agent/0.1' --post-data='%p' '%u' "</tt>
  *
  * NOTE: This function uses the curl's default HTTP-POST Content-Type:
  * application/x-www-form-urlencoded which is the only option allowed
@@ -495,7 +572,7 @@ char *oauth_http_get (const char *u, const char *q);
  * to transmit custom HTTP headers or parameters.
  *
  * WARNING: this is a tentative function. it's convenient and handy for testing
- * or developing oAuth code. But don't rely on this function
+ * or developing OAuth code. But don't rely on this function
  * to become a stable part of this API. It does not do 
  * much error checking for one thing..
  *
@@ -505,6 +582,27 @@ char *oauth_http_get (const char *u, const char *q);
  */
 char *oauth_http_post (const char *u, const char *p);
 
+/**
+ * do a HTTP POST request, wait for it to finish 
+ * and return the content of the reply.
+ * (requires libcurl)
+ *
+ * It's equivalent to /ref oauth_http_post, but offers
+ * the possibility to specify a custom HTTP header and
+ * has no support for commandline-curl.
+ *
+ * If liboauth is compiled <b>without</b> libcurl this function
+ * always returns NULL.
+ *
+ * @param u url to query
+ * @param p postargs to send along with the HTTP request.
+ * @param customheader specify custom HTTP header (or NULL for none)
+ * Multiple header elements can be passed separating them with "\r\n"
+ * @return replied content from HTTP server. needs to be freed by caller.
+ */
+char *oauth_http_post2 (const char *u, const char *p, const char *customheader);
+
+
 /**
  * http post raw data from file.
  * the returned string needs to be freed by the caller
@@ -515,7 +613,8 @@ char *oauth_http_post (const char *u, const char *p);
  * @param u url to retrieve
  * @param fn filename of the file to post along
  * @param len length of the file in bytes. set to '0' for autodetection
- * @param customheader specify custom HTTP header (or NULL for default)
+ * @param customheader specify custom HTTP header (or NULL for default).
+ * Multiple header elements can be passed separating them with "\r\n"
  * @return returned HTTP reply or NULL on error
  */
 char *oauth_post_file (const char *u, const char *fn, size_t len, const char *customheader);
@@ -531,6 +630,7 @@ char *oauth_post_file (const char *u, const char *fn, size_t len, const char *cu
  * @param data data to post
  * @param len length of the data in bytes. 
  * @param customheader specify custom HTTP header (or NULL for default)
+ * Multiple header elements can be passed separating them with "\r\n"
  * @return returned HTTP reply or NULL on error
  */
 char *oauth_post_data (const char *u, const char *data, size_t len, const char *customheader);
@@ -551,6 +651,7 @@ char *oauth_post_data (const char *u, const char *data, size_t len, const char *
  * @param data data to post along
  * @param len length of the file in bytes. set to '0' for autodetection
  * @param customheader specify custom HTTP header (or NULL for default)
+ * Multiple header elements can be passed separating them with "\r\n"
  * @param callback specify the callback function
  * @param callback_data specify data to pass to the callback function
  * @return returned HTTP reply or NULL on error
@@ -575,6 +676,7 @@ char *oauth_post_data_with_callback      (const char *u,
  * @param data data to post
  * @param len length of the data in bytes. 
  * @param customheader specify custom HTTP header (or NULL for default)
+ * Multiple header elements can be passed separating them with "\r\n"
  * @param httpMethod specify http verb ("GET"/"POST"/"PUT"/"DELETE") to be used. if httpMethod is NULL, a POST is executed.
  * @return returned HTTP reply or NULL on error
  */
@@ -600,6 +702,7 @@ char *oauth_send_data (const char *u,
  * @param data data to post along
  * @param len length of the file in bytes. set to '0' for autodetection
  * @param customheader specify custom HTTP header (or NULL for default)
+ * Multiple header elements can be passed separating them with "\r\n"
  * @param callback specify the callback function
  * @param callback_data specify data to pass to the callback function
  * @param httpMethod specify http verb ("GET"/"POST"/"PUT"/"DELETE") to be used.
